#!/bin/bash

# Repo Master Parameters
masterHostname=azure-apt-cat.cloudapp.net
masterReporoot=/home/azureuser/apt-catalog/repo_build/repos
masterAccounts=/home/azureuser/apt-catalog/repo_build/repo_accounts.csv
unlockFile=/home/azureuser/apt-catalog/repo_build/unlocked

# Local Parameters
localUser=azureuser
accounts=/etc/azuremirror/repo_accounts.csv
tmpAccounts=$accounts.tmp
reporoot=/var/reporoot
sshdConfig=/etc/ssh/sshd_config
sshKeyFile=/root/.ssh/reposync_rsa

newAccts=0
runningUser=$USER
syslogFacility=syslog
syslogTag=SyncMirror

# Function Definitions
function LogMessage {
    msg=$1
    priority=$2
    if [ -z "$msg" ];then
        return
    fi
    
    if [ -z "$priority" ]; then
        priority=info
    fi
    
    logger -p "$syslogFacility.$priority" -t $SyncMirror -- $msg
    echo "[$priority]: $msg"
}

function Bail {
    msg=$1
    if [ -z "$msg" ];then
        msg="An unspecified error occurred"
    fi
    LogMessage $msg error
    exit 1
}

# Pre-execution check
runningUser=$SUDO_USER
if [[ -z $SUDO_USER && "$USER" -ne "root" ]];then
    Bail "Program must run under sudo or root"
fi

# If build operation is occurring, don't synchronize, could result in an incomplete copy
scp -i $sshKeyFile $localUser@$masterHostname:$unlockFile /tmp 2> /dev/null
if [ $? -ne 0 ]; then
    LogMessage "Cannot sync repos from $masterHostname because its repo is locked. Waiting for the following file to be created: $masterHostname:$unlockFile" "warning"
    exit
fi

# Get CSV file
install -o root -g root -m 600 /dev/null $tmpAccounts
scp -i $sshKeyFile $localUser@$masterHostname:$masterAccounts $tmpAccounts

if [[ $? -ne 0 || ! -f $tmpAccounts ]]; then
    Bail "File $tmpAccounts was not successfully copied from master"
fi


# Move CSV file to a location where users cannot meddle with it
mv $tmpAccounts $accounts
if [ -f $accounts ]; then
    LogMessage "Successfully retrieved $accounts"
else
    Bail "Failed to move $tmpAccounts to $accounts"
fi

while IFS=, read username groups folder sshkey
do
    if [ -z $username ];then
        continue
    fi
    acctExists=$(grep -c "^$username:" /etc/passwd)
    # Setup account if it doesn't already exist
    if [ $acctExists -lt 1 ]; then
        # Set up account
        LogMessage "Account $username doesn't exist. Creating it."
        useradd -d /home/$username -m -s /bin/false $username
        install -o $username -g $username -m 700 -d /home/$username/.ssh
        # Set up ssh key Create file with appropriate permissions first
        # Then fill in the appropriate keys, then overwrite the old keys file
        authKeyFile=/home/$username/.ssh/authorized_keys
        install -o $username -g $username -m 600 /dev/null $authKeyFile.tmp
        echo "ssh-rsa $sshkey" > $authKeyFile.tmp
        mv $authKeyFile.tmp  $authKeyFile
        
        # Set up SFTP permissions
        cat << EOF >> $sshdConfig

Match User $username
	ChrootDirectory $reporoot
	AllowTCPForwarding no
	X11Forwarding no
	ForceCommand internal-sftp

EOF
        let "newAccts += 1"
    fi
    # Update group membership
    groupsCmd=$(echo $groups | tr ':' ',')
    LogMessage "Setting group membership for $username to [$groupsCmd]"
    usermod -G "$groupsCmd" $username
done < $accounts

if [ $newAccts -gt 0 ]; then
    LogMessage "Added $newAccts accounts, must restart SSH daemon"
    service ssh restart >> /dev/null
    if [ $? -ne 0 ]; then
        Bail "Error while restarting SSH daemon"
    fi
    LogMessage "Successfully restarted SSH daemon"  
fi

# Rsync package files
startTime=$(date +%s)
rsync -agpz -e "ssh -i $sshKeyFile" $localUser@$masterHostname:$masterReporoot $reporoot
errCode=$?
endTime=$(date +%s)
elapsed=$((endTime-startTime))
if [ $errCode -ne 0 ];then
    Bail "Rsync failed after $elapsed seconds"
else
    LogMessage "Rsync succeeded after $elapsed seconds"
fi